<?php

namespace Schaffen\SerasaRemessa\Utils;

class RecordColumn {

    /**
     * Tipo Alfabetico
     */
    const TYPE_ALPHABETICAL = 'A';

    /**
     * Tipo numérico
     */
    const TYPE_NUMERIC = 'N';

    /**
     * Tipo alfabetico
     */
    const TYPE_ALPHANUMERIC = 'X';


    /**
     * 
     * @var string Tipo do valor da coluna 'A', 'N' ou 'X' 
     */
    private $type;

    /**
     * @var int Posição da coluna na string do registro
     */
    private $position;

    /**
     * @var int Tamanho da coluna na string do registro
     */
    private $size;

    /**
     * @var string | int | double Valor da coluna
     */
    private $value;

    /** 
     * @param string | int | double $value Valor da coluna
     * @param string $type Tipo do valor da coluna 'A', 'N' ou 'X' 
     * @param int $position Posição da coluna na string do registro
     * @param int $size Tamanho da coluna na string do registro
     */
    public function __construct($value, $type, $position, $size){
        $this->value = $value;
        $this->type = $type;
        $this->size = $size;
        $this->position = $position;
    }

    /**
     * Define o valor da coluna
     * @var string | int | double $value Valor da coluna
     */
    public function setValue($value) {
        if($this->value === null) {
            $this->value = $value;
        }
    }

    /**
     * @return string Valor formatado no padrão CNAB
     */
    public function getFormatedValue() {
        return Util::formatCnab($this->type, $this->value, $this->size);
    }

    /**
     * @return string Resto do valor quando ele ultrapassar o tamanho da coluna
     */
    public function getRest() {
        return substr(Util::upper(Util::normalizeChars($this->value)), $this->size);
    }

    /**
     * @param string | int | double Valor da coluna
     */
    public function getValue() {
        return $this->value;
    }

    /**
     * @return int Posição da coluna
     */
    public function getPosition() {
        return $this->position;
    }

    /**
     * @return int Posição real da coluna na string (position - 1)
     */
    public function getRealPosition() {
        return $this->position - 1;
    }
}